{
 "cells": [
  {
   "cell_type": "markdown",
   "id": "f6f51d13-60e2-478c-8e11-1dd61e1cc330",
   "metadata": {},
   "source": [
    "# Plotly スライダーウィジェット\n",
    "\n",
    "公開日 <time datetime=\"2025-01-13\">2025-01-13</time>\n",
    "\n",
    "Pythonのインタラクティブなグラフを描画できるライブラリPlotlyにて、スライダーウィジェットを追加する方法を解説します。\n",
    "\n",
    "Plotlyには、高水準のAPIであるPlotly Expressと呼ばれるものもありますが、この記事では細かい調節ができる`graph_objects`を対象としています。"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "b6e8c83e-4884-490a-a85b-352a61150a08",
   "metadata": {},
   "source": [
    "<!-- START MoshimoAffiliateEasyLink -->\n",
    "<script type=\"text/javascript\">\n",
    "(function(b,c,f,g,a,d,e){b.MoshimoAffiliateObject=a;\n",
    "b[a]=b[a]||function(){arguments.currentScript=c.currentScript\n",
    "||c.scripts[c.scripts.length-2];(b[a].q=b[a].q||[]).push(arguments)};\n",
    "c.getElementById(a)||(d=c.createElement(f),d.src=g,\n",
    "d.id=a,e=c.getElementsByTagName(\"body\")[0],e.appendChild(d))})\n",
    "(window,document,\"script\",\"//dn.msmstatic.com/site/cardlink/bundle.js?20220329\",\"msmaflink\");\n",
    "msmaflink({\"n\":\"pandas\\u0026Plotly 2D\\/3Dデータビジュアライゼーション実装ハンドブック\",\"b\":\"秀和システム\",\"t\":\"\",\"d\":\"https:\\/\\/m.media-amazon.com\",\"c_p\":\"\\/images\\/I\",\"p\":[\"\\/51buhQYpkrL._SL500_.jpg\",\"\\/41vnATKu5RL._SL500_.jpg\",\"\\/41vDFPPpE1L._SL500_.jpg\",\"\\/4150EmIK-VL._SL500_.jpg\",\"\\/41-G1+HL-lL._SL500_.jpg\",\"\\/41fwd4F77vL._SL500_.jpg\",\"\\/31LL+GnIWuL._SL500_.jpg\",\"\\/41XR6ul2enL._SL500_.jpg\",\"\\/41RCmqHCXuL._SL500_.jpg\",\"\\/41qtNs0Fe-L._SL500_.jpg\"],\"u\":{\"u\":\"https:\\/\\/www.amazon.co.jp\\/dp\\/479806890X\",\"t\":\"amazon\",\"r_v\":\"\"},\"v\":\"2.1\",\"b_l\":[{\"id\":1,\"u_tx\":\"Amazonで見る\",\"u_bc\":\"#f79256\",\"u_url\":\"https:\\/\\/www.amazon.co.jp\\/dp\\/479806890X\",\"a_id\":4672318,\"p_id\":170,\"pl_id\":27060,\"pc_id\":185,\"s_n\":\"amazon\",\"u_so\":1},{\"id\":2,\"u_tx\":\"楽天市場で見る\",\"u_bc\":\"#f76956\",\"u_url\":\"https:\\/\\/search.rakuten.co.jp\\/search\\/mall\\/pandas%26Plotly%202D%2F3D%E3%83%87%E3%83%BC%E3%82%BF%E3%83%93%E3%82%B8%E3%83%A5%E3%82%A2%E3%83%A9%E3%82%A4%E3%82%BC%E3%83%BC%E3%82%B7%E3%83%A7%E3%83%B3%E5%AE%9F%E8%A3%85%E3%83%8F%E3%83%B3%E3%83%89%E3%83%96%E3%83%83%E3%82%AF\\/\",\"a_id\":4672316,\"p_id\":54,\"pl_id\":27059,\"pc_id\":54,\"s_n\":\"rakuten\",\"u_so\":2}],\"eid\":\"IXitR\",\"s\":\"l\"});\n",
    "</script>\n",
    "<div id=\"msmaflink-IXitR\">リンク</div>\n",
    "<!-- MoshimoAffiliateEasyLink END -->"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "cbab4bd3-bb15-40c1-b463-ffb8d4454a64",
   "metadata": {},
   "source": [
    "## スライダーの基礎\n",
    "\n",
    "Plotlyの棒グラフ `go.Bar` を3つ用意し、スライダーで切り替える例を示します。\n",
    "スライダーを右に動かすと、棒グラフの数値が増えます。"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "id": "63a0db3c-425f-400c-bf85-714831d48642",
   "metadata": {},
   "outputs": [],
   "source": [
    "import plotly.graph_objects as go\n",
    "\n",
    "fig = go.Figure(\n",
    "    data=[\n",
    "        go.Bar(x=[\"Alice\", \"Bob\"], y=[1, 2], visible=True),\n",
    "        go.Bar(x=[\"Alice\", \"Bob\"], y=[2, 4], visible=False),\n",
    "        go.Bar(x=[\"Alice\", \"Bob\"], y=[3, 6], visible=False),\n",
    "    ],\n",
    ")\n",
    "\n",
    "steps = []\n",
    "for i in range(len(fig.data)):\n",
    "    step = dict(\n",
    "        method=\"update\",\n",
    "        args=[{\"visible\": [False] * len(fig.data)}],\n",
    "        label=f\"{i}\"\n",
    "    )\n",
    "    step[\"args\"][0][\"visible\"][i] = True  # Toggle i'th trace to \"visible\"\n",
    "    steps.append(step)\n",
    "\n",
    "sliders = [dict(\n",
    "    active=0,\n",
    "    currentvalue={\"prefix\": \"Year: \"},\n",
    "    pad={\"t\": 30},\n",
    "    steps=steps\n",
    ")]\n",
    "\n",
    "fig.update_layout(sliders=sliders)\n",
    "fig.update_yaxes(range=(0, 8))\n",
    "fig.show()"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "2d3a776a-edb7-46ae-b58c-9f4811ff3430",
   "metadata": {},
   "source": [
    "上記の例を解説します。\n",
    "\n",
    "まず、スライダーで表示したいグラフ（ここでは`go.Bar`）をリストで`go.Figure`の`data`属性に定義します。\n",
    "なお、初期状態で表示したいグラフのみ`visible=True`として、それ以外のグラフは`visible=False`としておきます。\n",
    "\n",
    "```python\n",
    "fig = go.Figure(\n",
    "    data=[\n",
    "        go.Bar(x=[\"Alice\", \"Bob\"], y=[1, 2], visible=True),\n",
    "        go.Bar(x=[\"Alice\", \"Bob\"], y=[2, 4], visible=False),\n",
    "        go.Bar(x=[\"Alice\", \"Bob\"], y=[3, 6], visible=False),\n",
    "    ],\n",
    ")\n",
    "```\n"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "3770a8a0-0912-4121-b9b2-f0cb0ff8d7dc",
   "metadata": {},
   "source": [
    "次に、スライダーを動かしたときの挙動を以下のように変数`steps`に定義します。\n",
    "上記の例では、`len(fig.data)`は`3`となります。\n",
    "\n",
    "いったん全グラフの`visible`属性を`False`にした後、`i`番目のグラフ（スライダーで指定するグラフ）のみ`visible`属性を`True`とします。\n",
    "また、`label`属性には、スライダーの下部に表示するラベルを指定します（この例では`0`, `1`, `2`となる）。\n",
    "\n",
    "```python\n",
    "steps = []\n",
    "for i in range(len(fig.data)):\n",
    "    step = dict(\n",
    "        method=\"update\",\n",
    "        args=[{\"visible\": [False] * len(fig.data)}],\n",
    "        label=f\"{i}\"\n",
    "    )\n",
    "    step[\"args\"][0][\"visible\"][i] = True  # Toggle i'th trace to \"visible\"\n",
    "    steps.append(step)\n",
    "```\n"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "5b657629-a76d-441d-bb98-1320d056867d",
   "metadata": {},
   "source": [
    "さらに、以下のようにグラフにスライダーを追加します。\n",
    "\n",
    "```python\n",
    "sliders = [dict(\n",
    "    active=0,\n",
    "    currentvalue={\"prefix\": \"Year: \"},\n",
    "    pad={\"t\": 30},\n",
    "    steps=steps\n",
    ")]\n",
    "\n",
    "fig.update_layout(sliders=sliders)\n",
    "```\n",
    "\n",
    "`sliders`変数の辞書の意味は以下の通りです。\n",
    "\n",
    "- `active`: スライダーの初期位置を0始まりの整数で定義\n",
    "- `currentvalue`: スライダーで選択中の値に関する設定を定義\n",
    "  - `prefix`: 選択中の値の先頭に付ける文字列を定義\n",
    "- `pad`: スライダーの余白を定義\n",
    "- `step`: スライダーを動かしたときの挙動\n"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "ec6ef7e0-717d-47c4-9a39-449afbea3316",
   "metadata": {},
   "source": [
    "## スライダーの挙動\n",
    "\n",
    "スライダーを動かしたときの挙動に関する設定方法について、もう少し詳しく解説します。\n",
    "\n",
    "```python\n",
    "steps = []\n",
    "for i in range(len(fig.data)):\n",
    "    step = dict(\n",
    "        method=\"update\",\n",
    "        args=[{\"visible\": [False] * len(fig.data)}],\n",
    "        label=f\"{i}\"\n",
    "    )\n",
    "    step[\"args\"][0][\"visible\"][i] = True  # Toggle i'th trace to \"visible\"\n",
    "    steps.append(step)\n",
    "```\n",
    "\n",
    "上記のコードの`args`には、表示するグラフ以外についても設定できます。\n",
    "例えば、スライダーの位置に合わせてグラフのタイトルを変更する場合、以下のように書けます。\n",
    "\n",
    "```python\n",
    "args=[\n",
    "    {\"visible\": [False] * len(fig.data)},\n",
    "    {\"title\": f\"Alice & Bob's value at Year {i}\"},\n",
    "],\n",
    "```\n"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "2704b210-fe16-4eec-9e00-dd0a2362c452",
   "metadata": {},
   "source": [
    "## スライダーの表示\n",
    "\n",
    "スライダーの表示に関する設定を詳しく解説します。\n",
    "\n",
    "```python\n",
    "sliders = [dict(\n",
    "    active=0,\n",
    "    currentvalue={\"prefix\": \"Year: \"},\n",
    "    pad={\"t\": 30},\n",
    "    steps=steps\n",
    ")]\n",
    "```\n",
    "\n",
    "この`dict`には以下の値を設定できます。\n",
    "\n",
    "- `activebgcolor`: スライダーがアクティブな場合の色\n",
    "- `bgcolor`: スライダーの色\n",
    "- `bordercolor`: スライダーの枠線の色\n",
    "- `borderwidth`: スライダーの枠線の太さ\n",
    "- `currentvalue`: 選択中の値に関する設定\n",
    "  - `font`: フォントの設定\n",
    "    - `color`: 文字色\n",
    "  - `prefix`: 選択中の値の先頭に付ける文字列\n",
    "  - `suffix`: 選択中の値の後に付ける文字列\n",
    "- `font`: スライダー目盛りのフォント設定\n",
    "  - `color`: 文字色\n",
    "- `len`: スライダーの長さ（デフォルト値：`1`）\n",
    "- `lenmode`: スライダーの長さ`len`を指定するモード。`fraction`（デフォルト）、`pixels`から選択\n",
    "\n",
    "以下に例を示します。"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "id": "37987245-7599-4aef-866f-fdb75ebd0761",
   "metadata": {},
   "outputs": [],
   "source": [
    "sliders = [dict(\n",
    "    active=0,\n",
    "    bgcolor=\"red\",\n",
    "    bordercolor=\"black\",\n",
    "    currentvalue={\n",
    "        \"font\": {\"color\": \"green\"},\n",
    "        \"prefix\": \"Year: \",\n",
    "        \"suffix\": \" (Start: 2025)\",\n",
    "    },\n",
    "    font={\"color\": \"blue\"},\n",
    "    pad={\"t\": 30},\n",
    "    len=0.6,\n",
    "    steps=steps,\n",
    ")]\n",
    "\n",
    "fig.update_layout(sliders=sliders)\n",
    "fig.show()"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "2bac6786-49c8-432c-8561-f070aaafee80",
   "metadata": {},
   "source": [
    "## 参考サイト\n",
    "\n",
    "- [Sliders in Python - plotly](https://plotly.com/python/sliders/)\n",
    "- [Layout.sliders in Python - plotly](https://plotly.com/python/reference/layout/sliders/)"
   ]
  }
 ],
 "metadata": {
  "kernelspec": {
   "display_name": "Python 3 (ipykernel)",
   "language": "python",
   "name": "python3"
  },
  "language_info": {
   "codemirror_mode": {
    "name": "ipython",
    "version": 3
   },
   "file_extension": ".py",
   "mimetype": "text/x-python",
   "name": "python",
   "nbconvert_exporter": "python",
   "pygments_lexer": "ipython3",
   "version": "3.12.4"
  }
 },
 "nbformat": 4,
 "nbformat_minor": 5
}
